<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Category;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\File;

class CategoryController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $categories = Category::all();
        $data = compact('categories');
        return view('admin.category.index')->with($data);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('admin.category.manage_category', ['id' => null]);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        // Validate request data
        $validatedData = $request->validate([
            'ctg_name' => 'required',
            'ctg_banner' => 'image|max:2048',
            'ctg_main_img' => 'image|max:2048',
            'alt_tag' => 'required',
            'slug' => 'required|unique:categories',
            'meta_title' => 'required',
            'meta_description' => 'required',
        ], [
            'ctg_name.required' => 'The category name is required.',
            'ctg_banner.max' => 'The banner image size cannot exceed 2MB.',
            'ctg_main_img.max' => 'The main image size cannot exceed 2MB.',
            'alt_tag.required' => 'The alt tag is required.',
            'slug.required' => 'The slug is required.',
            'slug.unique' => 'The slug has already been taken.',
            'meta_title.required' => 'The meta title is required.',
            'meta_description.required' => 'The meta description is required.',
        ]);

        // Handle file upload
        if ($request->hasFile('ctg_banner')) {
            $file = $request->file('ctg_banner');
            $imageName = Str::slug($request->ctg_name) . '_' . now()->format('YmdHis') . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('uploads/category/banner'), $imageName);
        } else {
            // Handle case where file is not uploaded
            return redirect()->back()->withErrors(['ctg_banner' => 'The category banner is required.']);
        }
        if ($request->hasFile('ctg_main_img')) {
            $file = $request->file('ctg_main_img');
            $mainImageName = Str::slug($request->ctg_name) . '_' . now()->format('YmdHis') . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('uploads/category/main'), $mainImageName);
        } else {
            // Handle case where file is not uploaded
            return redirect()->back()->withErrors(['ctg_main_img' => 'The category main image is required.']);
        }

        // Create the Category
        $category = Category::create([
            'ctg_name' => $validatedData['ctg_name'],
            'ctg_banner' => $imageName,
            'ctg_main_img' => $mainImageName,
            'alt_tag' => $validatedData['alt_tag'],
            'slug' => Str::slug($validatedData['slug']),
            'meta_title' => $validatedData['meta_title'],
            'meta_description' => $validatedData['meta_description'],
        ]);

        // Redirect back with success message
        return redirect()->route('category')->with('success', 'Category added successfully.');
    }

    /**
     * Display the specified resource.
     */
    public function show($id)
    {
        $category = Category::findOrFail($id);
        $data = compact('category');
        return view('admin.category.manage_category')->with($data);
    }

    public function viewCtg($id)
    {
        $category = Category::findOrFail($id);
        
        return view('admin.category.view', ['category' => $category]);
    }
    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $id)
    {
        $request->validate([
            'ctg_name' => 'required',
            'ctg_banner' => 'image|max:2048',
            'ctg_main_img' => 'image|max:2048',
            'alt_tag' => 'required',
            'slug' => 'required|unique:categories,slug,'.$id,
            'meta_title' => 'required',
            'meta_description' => 'required',
        ], [
            'ctg_name.required' => 'The category name is required.',
            'ctg_banner.max' => 'The banner image size cannot exceed 2MB.',
            'ctg_main_img.max' => 'The main image size cannot exceed 2MB.',
            'alt_tag.required' => 'The alt tag is required.',
            'slug.required' => 'The slug is required.',
            'slug.unique' => 'The slug has already been taken.',
            'meta_title.required' => 'The meta title is required.',
            'meta_description.required' => 'The meta description is required.',
        ]);
        $category = Category::findOrFail($id);
        $category->update([
            'ctg_name' => $request->ctg_name,
            'alt_tag' => $request->alt_tag,
            'slug' => Str::slug($request->slug),
            'meta_title' => $request->meta_title,
            'meta_description' => $request->meta_description,
            'popular' => $request->has('popular'),
        ]);

            // Check if a new category banner is uploaded
    if ($request->hasFile('ctg_banner')) {
        $file = $request->file('ctg_banner');

        // Generate a new filename
        $imageName = Str::slug($request->ctg_name) . '_' . now()->format('YmdHis') . '.' . $file->getClientOriginalExtension();

        // Move the uploaded file to the desired location
        $file->move(public_path('uploads/category/banner/'), $imageName);

        // Delete the old banner image if it exists
        if ($category->ctg_banner && file_exists(public_path('uploads/category/banner/' . $category->ctg_banner))) {
            unlink(public_path('uploads/category/banner/' . $category->ctg_banner));
        }

        // Update the category banner path
        $category->ctg_banner = $imageName;
        $category->save();
    }

    // Check if a new sub category main image is uploaded
    if ($request->hasFile('ctg_main_img')) {
        $file = $request->file('ctg_main_img');

        // Generate a new filename
        $imageName = Str::slug($request->ctg_name) . '_main_' . now()->format('YmdHis') . '.' . $file->getClientOriginalExtension();

        // Move the uploaded file to the desired location
        $file->move(public_path('uploads/category/main/'), $imageName);

        // Delete the old sub category main image if it exists
        if ($category->ctg_main_img && file_exists(public_path('uploads/category/main/' . $category->ctg_main_img))) {
            unlink(public_path('uploads/category/main/' . $category->ctg_main_img));
        }
            // Update the sub category main image path
            $category->ctg_main_img = $imageName;
            $category->save();
        }

        return redirect()->route('category')->with('success', 'Category Updated successfully.');
    }


    /**
     * Remove the specified resource from storage.
     */
    public function destroy($id)
    {
        $category = Category::findOrFail($id);

        // Delete the category banner if it exists
        if ($category->ctg_banner) {
            unlink(public_path('uploads/category/banner/' . $category->ctg_banner));
        }

        // Delete the subcategory main image if it exists
        if ($category->ctg_main_img) {
            unlink(public_path('uploads/category/main/' . $category->ctg_main_img));
        }

        // Delete the category
        $category->delete();

        return redirect()->route('category')->with('success', 'Category deleted successfully.');
    }


}
