<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\SubCategory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\File;
use App\Models\Product;



class ProductController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $products = Product::all()->sortByDesc('created_at');
        $data = compact('products');
        return view('admin.product.index')->with($data);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $categories = Category::all();
        $subcategories = SubCategory::all();
        return view('admin.product.add_product', [
            'categories' => $categories,
            'subcategories' => $subcategories,
        ]);
    }




    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {

        $request->validate([
            'category_id' => 'array',
            'sub_category_id' => 'array',
            'product_name' => 'required|string|max:255',
            'product_desc' => 'required|string',
            'product_main_img' => 'required|image|max:2048', // Adjust image validation rules as needed
            'product_multi_img.*' => 'image|max:2048', // Validate each multi-image file
            'alt_tag' => 'required|string|max:255',
            'slug' => 'required|string|unique:products',
            'meta_title' => 'required|string|max:255',
            'meta_description' => 'required|string|max:255',
            'short_desc' => 'required|string'
        ], [
            'category_id.required' => 'The category field is required.',
            'sub_category_id.required' => 'The sub category field is required.',
            'product_name.required' => 'The product name field is required.',
            'product_desc.required' => 'The product description field is required.',
            'product_main_img.required' => 'The main image is required.',
            'product_main_img.image' => 'The main image must be an image.',
            'product_main_img.mimes' => 'The main image must be a file of type: jpeg, png, jpg, gif.',
            'product_main_img.max' => 'The main image may not be greater than 2048 kilobytes.',
            'product_multi_img.*.image' => 'The multi-image must be an image.',
            'product_multi_img.*.mimes' => 'The multi-image must be a file of type: jpeg, png, jpg, gif.',
            'product_multi_img.*.max' => 'The multi-image may not be greater than 2048 kilobytes.',
            'alt_tag.required' => 'The alt tag field is required.',
            'slug.required' => 'The slug field is required.',
            'slug.unique' => 'The slug has already been taken.',
            'meta_title.required' => 'The meta title field is required.',
            'short_desc.required' => 'The short description field is required.',
            'meta_description.required' => 'The meta description field is required.',
        ]);

        // Handle file uploads for main image
        $mainImageName = null;
        if ($request->hasFile('product_main_img')) {
            $file = $request->file('product_main_img');
            $mainImageName = Str::slug($request->input('product_name')) . '_' . now()->format('YmdHis') . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('uploads/product/main'), $mainImageName);
        }

        // Handle file uploads for multi images
        $multiImages = [];
        if ($request->hasFile('product_multi_img')) {
            foreach ($request->file('product_multi_img') as $multiImage) {
                $multiImageName = Str::slug($request->input('product_name')) . '_' . now()->format('YmdHis') . '.' . $multiImage->getClientOriginalExtension();
                $multiImage->move(public_path('uploads/product/multi'), $multiImageName);
                $multiImages[] = $multiImageName;
            }
        }
        $categoryIds = json_encode($request->input('category_id'));
        $subCategoryIds = json_encode($request->input('sub_category_id'));

        $product = Product::create([
            'category_id' => $categoryIds,
            'sub_category_id' => $subCategoryIds,
            'product_name' => $request->input('product_name'),
            'short_desc' => $request->input('short_desc'),
            'product_desc' => $request->input('product_desc'),
            'product_main_img' => $mainImageName,
            'product_multi_img' => implode(',', $multiImages),
            'alt_tag' => $request->input('alt_tag'),
            'slug' => Str::slug($request->input('slug')),
            'meta_title' => $request->input('meta_title'),
            'meta_description' => $request->input('meta_description'),
            'popular' => $request->input('popular', false),
        ]);

        if ($request->has('category_id')) {
            foreach ($request->input('category_id') as $categoryId) {
                $product->category()->attach($categoryId);
            }
        }
    
        // Attach subcategories to the product
        if ($request->has('sub_category_id')) {
            foreach ($request->input('sub_category_id') as $subCategoryId) {
                $product->subCategory()->attach($subCategoryId);
            }
        }
    
        // Redirect back with success message
        return redirect()->route('product')->with('success', 'Product added successfully.');
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        $product = Product::findOrFail($id);
        $categories = Category::all();
        $subcategories = SubCategory::all();
        $categoryIds = json_decode($product->category_id);
        $subCategoryIds = json_decode($product->sub_category_id);
        return view('admin.product.edit_product', [
            'categories' => $categories,
            'subcategories' => $subcategories,
            'categoryIds' => $categoryIds,
            'subCategoryIds' => $subCategoryIds,
            'product' => $product
        ]);
    }

    public function viewProduct(string $id)
    {
        $product = Product::findOrFail($id);
        // dd($product);
        $categories = Category::all();
        $subcategories = SubCategory::all();
        return view('admin.product.view', [
            'categories' => $categories,
            'subcategories' => $subcategories,
            'product' => $product
        ]);
    }


    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $id)
    {


        // Find the product by its ID
        $product = Product::findOrFail($id);

        // Handle file uploads for main image
        if ($request->hasFile('product_main_img')) {
            $file = $request->file('product_main_img');
            $mainImageName = Str::slug($request->input('product_name')) . '_' . now()->format('YmdHis') . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('uploads/product/main'), $mainImageName);
            // Delete the old main image if exists
            if ($product->product_main_img) {
                // Remove old main image file
                // Make sure to import the File facade: `use Illuminate\Support\Facades\File;`
                File::delete(public_path('uploads/product/main/' . $product->product_main_img));
            }
            $product->product_main_img = $mainImageName;
        }

        // Handle file uploads for multi images
        $multiImages = [];
        if ($request->hasFile('product_multi_img')) {
            foreach ($request->file('product_multi_img') as $multiImage) {
                $multiImageName = Str::slug($request->input('product_name')) . '_' . now()->format('YmdHis') . '.' . $multiImage->getClientOriginalExtension();
                $multiImage->move(public_path('uploads/product/multi'), $multiImageName);
                $multiImages[] = $multiImageName;
            }
            // Delete the old multi images if exist
            if ($product->product_multi_img) {
                // Split the old multi image names by comma and delete each file
                $oldMultiImages = explode(',', $product->product_multi_img);
                foreach ($oldMultiImages as $oldMultiImage) {
                    File::delete(public_path('uploads/product/multi/' . $oldMultiImage));
                }
            }
            $product->product_multi_img = implode(',', $multiImages);
        }
        $categoryIds = json_encode($request->input('category_id'));
        $subCategoryIds = json_encode($request->input('sub_category_id'));
        // Update the product with the new data
        $product->update([
            'category_id' => $categoryIds,
            'sub_category_id' => $subCategoryIds,
            'product_name' => $request->input('product_name'),
            'short_desc' => $request->input('short_desc'),
            'product_desc' => $request->input('product_desc'),
            'alt_tag' => $request->input('alt_tag'),
            'slug' => Str::slug($request->input('slug')),
            'meta_title' => $request->input('meta_title'),
            'meta_description' => $request->input('meta_description'),
            'popular' => $request->input('popular', false),
        ]);

    // Sync categories with the product
    $product->category()->sync($request->input('category_id', []));

    // Sync subcategories with the product
    $product->subCategory()->sync($request->input('sub_category_id', []));

        // Redirect back with success message
        return redirect()->route('product')->with('success', 'Product updated successfully.');
    }



    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        $product = Product::findOrFail($id);

        // Delete product banner from storage
        Storage::delete('uploads/product/main/' . $product->product_main_img);
        Storage::delete('uploads/product/multi/' . $product->product_multi_img);

        // Delete product
        $product->delete();

        // Redirect back with success message
        return redirect()->route('product')->with('success', 'Product deleted successfully.');
    }

    public function updateStatus(Request $request)
    {
        $request->validate([
            'product_id' => 'required|exists:products,id',
        ]);
        $product = Product::findOrFail($request->product_id);
        $product->update(['popular' => $request->popular ? 1 : 0]);

        // dd($product);
        return response()->json(['success' => true]);
    }

    public function getSubcategories(Request $request)
    {
        $categoryId = $request->input('category_id');

        // Retrieve subcategories based on the provided category ID
        $subcategories = SubCategory::where('ctg_id', $categoryId)->get();

        // Prepare subcategories data
        $data = [];
        foreach ($subcategories as $subcategory) {
            $data[$subcategory->id] = $subcategory->sub_ctg_name;
        }

        // Return subcategories as JSON response
        return response()->json($data);
    }


}
