<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Category;
use App\Models\SubCategory;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\File;

class SubCategoryController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $sub_categories = SubCategory::all()->sortByDesc('created_at');
        $data = compact('sub_categories');
        // dd($data);
        return view('admin.sub_category.index')->with($data);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $categories = Category::all();

        return view('admin.sub_category.manage_sub_category', ['id' => null, 'categories' => $categories]);
    }


    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        // Validate request data
        $validatedData = $request->validate([
            'ctg_id' => 'required|exists:categories,id',
            'sub_ctg_name' => 'required',
            'sub_ctg_banner' => 'image|max:2048',
            'sub_ctg_main_img' => 'image|max:2048',
            'alt_tag' => 'required',
            'slug' => 'required|unique:sub_categories',
            'meta_title' => 'required',
            'meta_description' => 'required',
        ], [
            'sub_ctg_name.required' => 'The category name is required.',
            'ctg_id.required' => 'The category is required.',
            'ctg_banner.max' => 'The banner image size cannot exceed 2MB.',
            'ctg_main_img.max' => 'The main image size cannot exceed 2MB.',
            'alt_tag.required' => 'The alt tag is required.',
            'slug.required' => 'The slug is required.',
            'slug.unique' => 'The slug has already been taken.',
            'meta_title.required' => 'The meta title is required.',
            'meta_description.required' => 'The meta description is required.',
        ]);

        // Handle file upload
        if ($request->hasFile('sub_ctg_banner')) {
            $file = $request->file('sub_ctg_banner');
            $imageName = Str::slug($request->sub_ctg_name) . '_' . now()->format('YmdHis') . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('uploads/sub_category/banner'), $imageName);
        } else {
            // Handle case where file is not uploaded
            return redirect()->back()->withErrors(['sub_ctg_banner' => 'The category banner is required.']);
        }
        if ($request->hasFile('sub_ctg_main_img')) {
            $file = $request->file('sub_ctg_main_img');
            $mainImageName = Str::slug($request->sub_ctg_name) . '_' . now()->format('YmdHis') . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('uploads/sub_category/main'), $mainImageName);
        } else {
            // Handle case where file is not uploaded
            return redirect()->back()->withErrors(['sub_ctg_main_img' => 'The category main image is required.']);
        }
        // Create the Subcategory
        $sub_category = SubCategory::create([
            'ctg_id' => $validatedData['ctg_id'],
            'sub_ctg_name' => $validatedData['sub_ctg_name'],
            'sub_ctg_main_img' => $mainImageName,
            'sub_ctg_banner' => $imageName,
            'alt_tag' => $validatedData['alt_tag'],
            'slug' => Str::slug($validatedData['slug']),
            'meta_title' => $validatedData['meta_title'],
            'meta_description' => $validatedData['meta_description'],
        ]);

        // Redirect back with success message
        return redirect()->route('sub_category')->with('success', 'Sub category added successfully.');
    }

    /**
     * Display the specified resource.
     */
    public function show($id)
    {
        $sub_category = SubCategory::findOrFail($id);
        $categories = Category::all();
        return view('admin.sub_category.manage_sub_category', ['sub_category' => $sub_category, 'categories' => $categories]);
    }



    public function viewSubCtg($id)
    {
        $sub_category = SubCategory::findOrFail($id);
        $categories = Category::all();
        return view('admin.sub_category.view', ['sub_category' => $sub_category, 'categories' => $categories]);
    }


    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $id)
{
    // Validate request data
    $validatedData = $request->validate([
        'ctg_id' => 'required|exists:categories,id',
        'sub_ctg_name' => 'required',
        'sub_ctg_banner' => 'image|max:2048',
        'sub_ctg_main_img' => 'image|max:2048',
        'alt_tag' => 'required',
        'slug' => 'required|unique:categories,slug,' . $id,
        'meta_title' => 'required',
        'meta_description' => 'required',
    ], [
        // Validation messages...
    ]);

    // Find the subcategory
    $sub_category = SubCategory::findOrFail($id);

    // Handle file upload if a new banner is provided
    if ($request->hasFile('sub_ctg_banner')) {
        $file = $request->file('sub_ctg_banner');
        $imageName = Str::slug($validatedData['slug']) . '_' . now()->format('YmdHis') . '.' . $file->getClientOriginalExtension();
        $file->move(public_path('uploads/sub_category/banner'), $imageName);

        // Delete the old banner file if exists
        if ($sub_category->sub_ctg_banner) {
            $oldImagePath = public_path('uploads/sub_category/banner/' . $sub_category->sub_ctg_banner);
            if (file_exists($oldImagePath)) {
                unlink($oldImagePath);
            }
        }
        // Update subcategory with new banner
        $sub_category->sub_ctg_banner = $imageName;
    }

    // Handle file upload if a new main image is provided
    if ($request->hasFile('sub_ctg_main_img')) {
        $file = $request->file('sub_ctg_main_img');
        $mainImageName = Str::slug($validatedData['slug']) . '_' . now()->format('YmdHis') . '.' . $file->getClientOriginalExtension();
        $file->move(public_path('uploads/sub_category/main'), $mainImageName);

        // Delete the old main image file if exists
        if ($sub_category->sub_ctg_main_img) {
            $oldImagePath = public_path('uploads/sub_category/main/' . $sub_category->sub_ctg_main_img);
            if (file_exists($oldImagePath)) {
                unlink($oldImagePath);
            }
        }
        // Update subcategory with new main image
        $sub_category->sub_ctg_main_img = $mainImageName;
    }

    // Update the rest of the subcategory data
    $sub_category->ctg_id = $validatedData['ctg_id'];
    $sub_category->sub_ctg_name = $validatedData['sub_ctg_name'];
    $sub_category->alt_tag = $validatedData['alt_tag'];
    $sub_category->slug = Str::slug($validatedData['slug']);
    $sub_category->meta_title = $validatedData['meta_title'];
    $sub_category->meta_description = $validatedData['meta_description'];

    // Save the changes
    $sub_category->save();

    // Redirect back with success message
    return redirect()->route('sub_category')->with('success', 'Sub category updated successfully.');
}


    /**
     * Remove the specified resource from storage.
     */
    public function destroy($id)
{
    // Find the category by ID
    $sub_category = SubCategory::findOrFail($id);

    // Delete the sub_category banner if it exists
    if ($sub_category->sub_ctg_banner && file_exists(public_path('uploads/sub_category/banner/' . $sub_category->sub_ctg_banner))) {
        unlink(public_path('uploads/sub_category/banner/' . $sub_category->sub_ctg_banner));
    }

    // Delete the sub_category main image if it exists
    if ($sub_category->sub_ctg_main_img && file_exists(public_path('uploads/sub_category/main/' . $sub_category->sub_ctg_main_img))) {
        unlink(public_path('uploads/sub_category/main/' . $sub_category->sub_ctg_main_img));
    }

    // Delete the sub_category
    $sub_category->delete();

    return redirect()->back()->with('success', 'Sub Category deleted successfully.');
}

}
